import { Component } from '@angular/core';
import {DatePipe, NgForOf, NgIf} from "@angular/common";
import {DictionaryPipe} from "../../../pipes/common/dictionary.pipe";
import {FormGroup, FormsModule, NonNullableFormBuilder, ReactiveFormsModule, Validators} from "@angular/forms";
import {NzButtonComponent} from "ng-zorro-antd/button";
import {
  NzCellFixedDirective,
  NzTableCellDirective,
  NzTableComponent, NzTableModule,
  NzTbodyComponent, NzTheadComponent,
  NzThMeasureDirective, NzTrDirective
} from "ng-zorro-antd/table";
import {NzColDirective, NzRowDirective} from "ng-zorro-antd/grid";
import {NzDividerComponent} from "ng-zorro-antd/divider";
import {NzDropDownADirective, NzDropDownDirective, NzDropdownMenuComponent} from "ng-zorro-antd/dropdown";
import {NzFormControlComponent, NzFormDirective, NzFormItemComponent, NzFormLabelComponent} from "ng-zorro-antd/form";
import {NzIconDirective} from "ng-zorro-antd/icon";
import {NzInputDirective} from "ng-zorro-antd/input";
import {NzMenuDirective, NzMenuItemComponent} from "ng-zorro-antd/menu";
import {NzOptionComponent, NzSelectComponent} from "ng-zorro-antd/select";
import {NzMessageService} from "ng-zorro-antd/message";
import {NzModalModule, NzModalService} from "ng-zorro-antd/modal";
import {NzFlexDirective} from "ng-zorro-antd/flex";
import {NzDescriptionsModule} from "ng-zorro-antd/descriptions";
import {NzUploadComponent, NzUploadFile} from "ng-zorro-antd/upload";
import {environment} from "../../../../environments/environment";
import {Observable, Observer} from "rxjs";
import {NzAvatarModule} from "ng-zorro-antd/avatar";
import {MerService} from "../../../services/merchant/mer.service";
import {RoleService} from "../../../services/role/role.service";
import {NzImageModule} from "ng-zorro-antd/image";
import {NzSwitchComponent} from "ng-zorro-antd/switch";
import {DateSelectType} from "../../../utils/dateUtils.service";
import {NzDatePickerComponent, NzRangePickerComponent} from "ng-zorro-antd/date-picker";
import {NzRadioButtonDirective, NzRadioComponent, NzRadioGroupComponent} from "ng-zorro-antd/radio";
import {CommonService} from "../../../services/common/common.service";
import { NzToolTipModule } from 'ng-zorro-antd/tooltip';
@Component({
  selector: 'app-mer-list',
  standalone: true,
  imports: [
    DatePipe,
    DictionaryPipe,
    FormsModule,
    NgForOf,
    NgIf,
    NzButtonComponent,
    NzCellFixedDirective,
    NzColDirective,
    NzDividerComponent,
    NzDropDownADirective,
    NzDropDownDirective,
    NzDropdownMenuComponent,
    NzFormControlComponent,
    NzFormDirective,
    NzFormItemComponent,
    NzFormLabelComponent,
    NzIconDirective,
    NzInputDirective,
    NzMenuDirective,
    NzMenuItemComponent,
    NzOptionComponent,
    NzRowDirective,
    NzSelectComponent,
    NzTableCellDirective,
    NzTableComponent,
    NzTbodyComponent,
    NzThMeasureDirective,
    NzTheadComponent,
    NzTrDirective,
    ReactiveFormsModule,
    NzTableModule,
    NzModalModule,
    NzFlexDirective,
    NzDescriptionsModule,
    NzUploadComponent,
    NzAvatarModule,
    NzImageModule,
    NzSwitchComponent,
    NzDatePickerComponent,
    NzRadioButtonDirective,
    NzRadioComponent,
    NzRadioGroupComponent,
    NzRangePickerComponent,
    NzToolTipModule
  ],
  templateUrl: './mer-list.component.html',
  styleUrl: './mer-list.component.less'
})
export class MerListComponent {
  BASE_URL = environment.baseUrl;
  IMAGE_URL = environment.imageUrl;
  // 表单页数
  tablePageNum = 1;
  // 每页数量
  tablePageSize = 10;
  // 表单数据
  tableData: any = {
    total: 0,
    list: [],
  };
  // 搜索表单
  searchForm: FormGroup;
  // 更多搜索条件
  isCollapse = false;
  // 油站状态
  merchantStatusArray = new DictionaryPipe().getDictionaryList('MERCHANT_STATUS');
  // 油站来源
  merSourceTypeArray = new DictionaryPipe().getDictionaryList('MER_SOURCE_TYPE');
  // 地区数据
  regionData: any = []
  roleData: any = []
  merDetailVisible = false;
  merDetailObject: any = {};


  merLogo = '';
  uploadLoading = false;
  updateMerVisible = false;
  updateMerForm: FormGroup;

  constructor(private fb: NonNullableFormBuilder,
              private merService: MerService,
              private roleService: RoleService,
              private message: NzMessageService,
              private commonService: CommonService,
              private modal: NzModalService) {
    // 初始化搜索框
    this.searchForm = this.fb.group({
      sourceType: [''],
      provinceCode: [''],
      merNo: [''],
      merName: [''],
      status: [''],
    });

    this.updateMerForm = this.fb.group({
      loginName: [{ value: '', disabled: true}, [Validators.required]],
      roleId: ['', [Validators.required]],
      merLogo: [''],
      merNo: [''],
      merName: ['', [Validators.required]],
      directorName: ['',[Validators.required]],
      directorTel: ['',[Validators.required]],
      customerServiceTel: [''],
      address: [''],
    });

    // 获取角色数据
    this.roleService.queryAllRole((data: any) => {
      this.roleData = data['return_data'];
    });
    // 获取地区
    this.commonService.getRegion("", (data: any) => {
      this.regionData = data['return_data'];
    });

    this.queryData();
  }

  /**
   * 获取数据
   */
  queryData() {
    this.searchForm.value.pageNum = this.tablePageNum;
    this.searchForm.value.pageSize = this.tablePageSize;
    this.searchForm.value.time = new Date().getTime();
    this.merService.queryMerList(this.searchForm.value, (data: any) => {
      if (data['return_code'] == '000000') {
        this.tableData = data['return_data'];
      }
    });
  }

  /**
   * 搜索表单提交
   */
  searchFormSubmit(): void {
    this.tablePageNum = 1;
    this.queryData();
  }

  /**
   * 搜索表单重置
   */
  searchFormReset(): void {
    this.searchForm.reset();
  }

  /**
   * 展示商户详情
   * @param merNo
   */
  showMerDetail(merNo: string) {
    this.merService.queryMer(merNo, (data: any) => {
      if (data['return_code'] == '000000') {
        this.merDetailObject = data['return_data'];
      } else {
        this.message.error(data['return_msg']);
      }
    });
    this.merDetailVisible = true;
  }

  /**
   * 关闭商户详情
   */
  closeMerDetail() {
    this.merDetailObject = {};
    this.merDetailVisible = false;
  }

  /**
   * 列表展示
   */
  listShowChange(data: boolean) {
    console.log(data);
  }

  /**
   * 打开修改商户模态框
   * @param merData
   */
  showUpdateMer(merNo: string) {
    this.merService.queryMer(merNo, (data: any) => {
      if (data['return_code'] == '000000') {
        data['return_data']['roleId'] = String(data['return_data']['roleId']);
        this.updateMerForm.patchValue(data['return_data']);

        console.log(this.updateMerForm.controls['merLogo'].value);
      } else {
        this.message.error(data['return_msg']);
      }
    });
    this.updateMerVisible = true;
  }

  /**
   * 提交修改商户
   */
  submitUpdateMerForm() {
    if (this.updateMerForm.valid) {
      this.merService.update(this.updateMerForm.value, (data: any) => {
        if (data['return_code'] == '000000') {

          this.message.create('success', '操作成功');

          // 关闭弹窗
          this.closeUpdateMer();
          // 刷新列表
          this.queryData();

        } else {
          this.message.create('error', data['return_msg']);
        }
      });
    } else {
      Object.values(this.updateMerForm.controls).forEach(control => {
        if (control.invalid) {
          control.markAsDirty();
          control.updateValueAndValidity({ onlySelf: true });
        }
      });
    }
  }

  /**
   * 关闭修改商户模态框
   */
  closeUpdateMer() {
    this.updateMerForm.reset();
    this.updateMerVisible = false;
  }

  /**
   * 打开密码重置确认框
   */
  showResetPwd(merNo: string) {
    this.modal.confirm({
      nzTitle: '提示',
      nzContent: '确定重置商户账户登录密码吗？',
      nzOnOk: () => this.resetPwd(merNo)
    });
  }

  /**
   * 密码重置
   */
  resetPwd(merNo: string) {
    const param = {
      merNo: merNo
    }
    this.merService.resetPwd(param, (data: any) => {
      if (data['return_code'] == '000000') {
        // 刷新数据
        this.queryData();
        this.modal.success({
          nzTitle: '提示',
          nzContent: '密码重置成功，新密码：123456'
        });
      } else {
        this.message.create('error', data['return_msg']);
      }
    });
  }

  private getBase64(img: File, callback: (img: string) => void): void {
    const reader = new FileReader();
    reader.addEventListener('load', () => callback(reader.result!.toString()));
    reader.readAsDataURL(img);
  }

  handleChange(info: { file: NzUploadFile }): void {
    switch (info.file.status) {
      case 'uploading':
        this.uploadLoading = true;
        break;
      case 'done':
        // Get this url from response in real world.
        this.getBase64(info.file!.originFileObj!, (img: string) => {
          this.uploadLoading = false;
          this.merLogo = img;
        });
        this.updateMerForm.patchValue({ merLogo: info.file.response['return_data'][0]});
        break;
      case 'error':
        this.message.error('网络错误！');
        this.uploadLoading = false;
        break;
    }
  }

  beforeUpload = (file: NzUploadFile, _fileList: NzUploadFile[]) => {
    return new Observable((observer: Observer<boolean>) => {
      const isJpgOrPng = file.type === 'image/jpeg' || file.type === 'image/png';
      if (!isJpgOrPng) {
        this.message.error('只能上传JPG文件!');
        observer.complete();
        return;
      }
      const isLt2M = file.size! / 1024 / 1024 < 2;
      if (!isLt2M) {
        this.message.error('图片必须小于2MB!');
        observer.complete();
        return;
      }
      observer.next(isJpgOrPng && isLt2M);
      observer.complete();
    });
  };

  protected readonly DateSelectType = DateSelectType;
}
